/* Module functions. */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <unistd.h>

#include "modulefunc.h"
#include "options.h"

/* A linked list of all the temp files. */
struct tmpfile *tmpfilelist = NULL;

/* Signals to try on a non-responding module. */
const int killsignal[] = {SIGINT,SIGHUP,SIGKILL};

/* Kills a module. */
void kill_module(pid_t mpid){
  int i,r,status;
  pid_t tmppid;
  
  if (!daemon_mode)
    printf("Killing module pid %d\n",mpid);

  for (i = 0; i < 4; i++){

    if (!daemon_mode)
      printf("Sending signal %d to module pid %d\n",killsignal[i],mpid);

    /* Send killsignal[i] to mpid. */
    r = kill(mpid,killsignal[i]);

    /* Wait for mpid to close. It should exit, but if not we can try another signal on it. */
    tmppid = waitpid(mpid,&status,WNOHANG);
    
    /* Did mpid exit? */
    if ((WIFEXITED(status) | WIFSIGNALED(status)) & (tmppid == mpid))
      break;

    /* If the child didn't exit wait a few seconds. */
    sleep(module_kill_signal_interval);
    
    /* Keep repeating the kill signal untill the module is dead. */
    if (i == 3)
      i = 2;
  }
}

/* Waits for a module to exit. Kills the module after timeout seconds. 
   Returns -1 if the module exited normally. 0 if it had to be killed.
*/
int wait_module(pid_t pid,int timeout){
  int status,i;
  pid_t rpid;

  for (i = 0; i < timeout; i++){
    sleep(1);

    /* Is the module still alive? */
    rpid = waitpid(pid,&status,WNOHANG);
    if (rpid == pid)
      return -1;
  }

  /* Looks like the module will have to be killed. Call kill_module(pid) */
  kill_module(pid);

  return 0;
}

/* Makes a temp name and adds it to the temp file list. */
char *mktempname(void){
  char *tmp;
  char *tmp2;
  struct tmpfile *tmptmpfile;

  tmp = tmpnam(NULL);
  tmp2 = malloc(strlen(tmp) + 1);
  strcpy(tmp2,tmp);

  /* Adds the temp file to the temp file list. */
  tmptmpfile = malloc(sizeof(struct tmpfile));
  tmptmpfile->name = tmp2;
  tmptmpfile->prev = NULL;

  if (tmpfilelist){
    tmptmpfile->next = tmpfilelist;
    tmpfilelist = tmptmpfile;
    tmpfilelist->next->prev = tmpfilelist;
  }
  else{
    tmptmpfile->next = NULL;
    tmpfilelist = tmptmpfile;
  }

  return tmpfilelist->name;
}

/* Removes all temp files. */
void rmalltmpfiles(void){
  struct tmpfile *curfile;

  curfile = tmpfilelist;
  while (1){
    if (!curfile)
      return;

    remove(curfile->name);

    if (curfile->next)
      curfile = curfile->next;
    else
      return;
  }
}

/* Removes a temp file. */
void rmtempfile(char *file){
  struct tmpfile *rmtmp,*curfile;

  /* Find which tmpfilelist tmpfile is file. */
  curfile = tmpfilelist;
  while (1){
    if (!strcmp(curfile->name,file)){
      rmtmp = curfile;
      break;
    }
    if (curfile->next)
      curfile = curfile->next;
    else
      return;
  }

  remove(rmtmp->name);

  if (rmtmp->next){
    if (rmtmp->prev)
      rmtmp->next->prev = rmtmp->prev;
    else
      rmtmp->next->prev = NULL;
  }
  if (rmtmp->prev){
    if (rmtmp->next)
      rmtmp->prev->next = rmtmp->next;
    else
      rmtmp->prev->next = NULL;
  }
  
  if (rmtmp == tmpfilelist){
    if (rmtmp->next)
      tmpfilelist = rmtmp->next;
    else if (rmtmp->prev)
      tmpfilelist = rmtmp->prev;
    else
      tmpfilelist = NULL;
  }

  free(rmtmp->name);
  free(rmtmp);
}
 
static char *id="@(#) $Id: modulefunc.c,v 4.0 1999/10/07 22:56:14 pete Exp $";

/* 
 * $Log: modulefunc.c,v $
 * Revision 4.0  1999/10/07 22:56:14  pete
 * Started work on version 1.0
 *
 * Revision 3.4  1999/09/21 02:09:42  pete
 * *** empty log message ***
 *
 * Revision 3.3  1999/09/21 00:43:46  pete
 * As usual I forgot to compile and found some trivial error. :)
 *
 * Revision 3.2  1999/09/21 00:43:10  pete
 * Added wait_module function.
 *
 * Revision 3.1  1999/09/19 18:25:56  pete
 * Added support for automake
 *
 * Revision 3.0  1999/09/19 01:15:23  pete
 * Started work on 0.03
 *
 * Revision 2.2  1999/09/17 23:41:26  pete
 * Fixed a string handling bug.
 *
 * Revision 2.1  1999/09/17 21:50:22  pete
 * Added debugging output to kill_module
 *
 * Revision 2.0  1999/09/15 16:05:52  pete
 * Started work on version 0.02
 *
 * Revision 1.11  1999/09/12 01:31:09  pete
 * Added function rmalltmpfiles
 *
 * Revision 1.10  1999/09/12 00:46:52  pete
 * Bug fix.
 *
 * Revision 1.9  1999/09/12 00:13:20  pete
 * Made rmtempfile remove all files if file is NULL.
 *
 * Revision 1.8  1999/09/12 00:03:57  pete
 * Changed mktempname to record the temp files created in a linked list.
 * rmtempfile then removes the entries in the linked list.
 *
 * Revision 1.7  1999/09/11 20:13:45  pete
 * And fixed things so they would compile. :)
 *
 * Revision 1.6  1999/09/11 20:13:16  pete
 * Added mktempname function.
 *
 * Revision 1.5  1999/09/11 02:13:03  pete
 * Removed some debugging printf's
 *
 * Revision 1.4  1999/09/11 02:07:34  pete
 * Added ID and LOG tags after much work. :)
 *
 */
